function class = nnclassify( sample, training, group, k, distance )
% just like matlab's knnclassify, except allows arbitrary distance functions
% as allowed by matlab's pdist function.
% distance can be:
% 
%        'euclidean'   - Euclidean distance
%        'seuclidean'  - Standardized Euclidean distance, each coordinate
%                        in the sum of squares is inverse weighted by the
%                        sample variance of that coordinate
%        'cityblock'   - City Block distance
%        'mahalanobis' - Mahalanobis distance
%        'minkowski'   - Minkowski distance with exponent 2
%        'cosine'      - One minus the cosine of the included angle
%                        between observations (treated as vectors)
%        'correlation' - One minus the sample linear correlation between
%                        observations (treated as sequences of values).
%        'spearman'    - One minus the sample Spearman's rank correlation
%                        between observations (treated as sequences of values).
%        'hamming'     - Hamming distance, percentage of coordinates
%                        that differ
%        'jaccard'     - One minus the Jaccard coefficient, the
%                        percentage of nonzero coordinates that differ
%        'chebychev'   - Chebychev distance (maximum coordinate difference)
%        function      - A distance function specified using @, for
%                        example @DISTFUN

num_sample = size(sample,1);
num_training = size(training,1);

% TODO: evaluate just top row(s), not all pairwise distances, for efficiency
% also, don't use squareform
distances = squareform( pdist( [ sample; training ], distance ) );

for i=1:num_sample
    % distances to the training samples
    training_distances = distances( num_sample+1:end, i );

    % sort so that we can find the closest training sample
    [y,ind] = sort(training_distances); 

    % give this sample the same group label as the closest training sample
    class(i) = group(ind(1));
end

% HERE IS MORE EFFICIENT CODE THAT REQUIRES VERSION 2010a
%
%distances = squareform( pdist2( sample, training, distance ) );
%
%for i=1:num_sample
%    % distances to the training samples
%    training_distances = distances( i, : )';
%
%    % sort so that we can find the closest training sample
%    [y,ind] = sort(training_distances);
%
%    % give this sample the same group label as the closest training sample
%    class(i) = group(ind(1));
%end

